"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isReactClass = exports.isStatelessComponent = exports.isRelease = exports.check = exports.isProptypesRemove = exports.isIgnore = exports.remove = exports.normalizeOptions = void 0;
const types_1 = require("@babel/types");
const traversed = Symbol("traversed");
function normalizeOptions(options) {
    const { imports, styles, propTypes } = options;
    if (!imports.customImports)
        imports.customImports = [];
    if (!Array.isArray(imports.customImports)) {
        imports.customImports = [imports.customImports];
    }
    if (!imports.ignoreLibraries)
        imports.ignoreLibraries = ["react"];
    if (!Array.isArray(imports.ignoreLibraries)) {
        imports.ignoreLibraries = [imports.ignoreLibraries];
    }
    imports.ignoreFilenames = imports.ignoreFilenames
        ? new RegExp(arrayToString(imports.ignoreFilenames), "i")
        : "";
    styles.ignoreFilenames = styles.ignoreFilenames
        ? new RegExp(arrayToString(styles.ignoreFilenames), "i")
        : "";
    propTypes.ignoreFilenames = propTypes.ignoreFilenames
        ? new RegExp(arrayToString(propTypes.ignoreFilenames), "i")
        : "";
}
exports.normalizeOptions = normalizeOptions;
function arrayToString(value) {
    if (typeof value === 'object') {
        return value.join('|');
    }
    else {
        return value;
    }
}
function remove(path) {
    if (path.parentPath.type === "ConditionalExpression") {
        path.replaceWith((0, types_1.unaryExpression)("void", (0, types_1.numericLiteral)(0)));
    }
    else {
        path.remove();
    }
}
exports.remove = remove;
function isIgnore(scope, regex) {
    var _a, _b, _c;
    return false;
    const filename = (_c = (_b = (_a = scope.hub) === null || _a === void 0 ? void 0 : _a.file) === null || _b === void 0 ? void 0 : _b.opts) === null || _c === void 0 ? void 0 : _c.filename;
    if (!filename) {
        return true;
    }
    if (!regex) {
        return false;
    }
    return regex.test(filename);
}
exports.isIgnore = isIgnore;
function isProptypesRemove(path, globalOptions) {
    const { remove, ignoreFilenames, onlyProduction } = globalOptions.propTypes;
    if (!remove || isIgnore(path.scope, ignoreFilenames)) {
        return false;
    }
    if (onlyProduction) {
        return process.env.NODE_ENV === "production";
    }
    return true;
}
exports.isProptypesRemove = isProptypesRemove;
function toSnakeCase(key) {
    return key.replace(/[a-z][A-Z]/g, (s) => s[0] + "_" + s[1]).toUpperCase();
}
function getParam(key, state) {
    let keyS = toSnakeCase(key);
    let result = state.opts.var[keyS];
    if (state.opts.var.hasOwnProperty(key)) {
        result = state.opts.var[key];
    }
    if (process.env[keyS] !== undefined) {
        result = process.env[keyS];
    }
    if (result === "true" ||
        result === true ||
        result === "1" ||
        result === 1) {
        return true;
    }
    if (result === "false" ||
        result === false ||
        result === "0" ||
        result === 0) {
        return false;
    }
    return false;
}
function check(node, state) {
    if ((0, types_1.isStringLiteral)(node)) {
        return getParam(node.value, state);
    }
    if ((0, types_1.isIdentifier)(node)) {
        return getParam(node.name, state);
    }
    if ((0, types_1.isUnaryExpression)(node) && node.operator == "!") {
        let r = check(node.argument, state);
        return r === undefined ? r : !r;
    }
    return false;
}
exports.check = check;
function isRelease(state) {
    return (state.opts.isRelease ||
        process.env.IS_RELEASE ||
        /release|production/gi.test(process.env.ENV || ""));
}
exports.isRelease = isRelease;
function isJSXElementOrReactCreateElement(path) {
    let visited = false;
    path.traverse({
        CallExpression(path2) {
            const callee = path2.get("callee");
            if (callee.matchesPattern("React.createElement") ||
                callee.matchesPattern("React.cloneElement") ||
                callee.node.name === "cloneElement") {
                visited = true;
            }
        },
        JSXElement() {
            visited = true;
        },
    });
    return visited;
}
function isReturningJSXElement(path, iteration = 0) {
    if (path.node.init &&
        path.node.init.body &&
        isJSXElementOrReactCreateElement(path)) {
        return true;
    }
    if (iteration > 20) {
        throw new Error("transform-react-remove-prop-type: infinite loop detected.");
    }
    let visited = false;
    path.traverse({
        ReturnStatement(path2) {
            if (visited) {
                return;
            }
            const argument = path2.get("argument");
            if (!argument.node) {
                return;
            }
            if (isJSXElementOrReactCreateElement(path2)) {
                visited = true;
                return;
            }
            if (argument.node.type === "CallExpression") {
                const name = argument.get("callee").node.name;
                const binding = path.scope.getBinding(name);
                if (!binding) {
                    return;
                }
                if (binding.path[traversed]) {
                    return;
                }
                binding.path[traversed] = true;
                if (isReturningJSXElement(binding.path, iteration + 1)) {
                    visited = true;
                }
            }
        },
    });
    return visited;
}
const VALID_POSSIBLE_STATELESS_COMPONENT_TYPES = [
    "VariableDeclarator",
    "FunctionDeclaration",
];
function isStatelessComponent(path) {
    if (VALID_POSSIBLE_STATELESS_COMPONENT_TYPES.indexOf(path.node.type) === -1) {
        return false;
    }
    if (isReturningJSXElement(path)) {
        return true;
    }
    return false;
}
exports.isStatelessComponent = isStatelessComponent;
function isPathReactClass(path) {
    const node = path.node;
    if (path.matchesPattern("React.Component") ||
        path.matchesPattern("React.PureComponent")) {
        return true;
    }
    if (node && (node.name === "Component" || node.name === "PureComponent")) {
        return true;
    }
    return false;
}
function isReactClass(superClass, scope) {
    if (!superClass.node) {
        return false;
    }
    let answer = false;
    if (isPathReactClass(superClass)) {
        answer = true;
    }
    else if (superClass.node.name) {
        const className = superClass.node.name;
        const binding = scope.getBinding(className);
        if (!binding) {
            answer = false;
        }
        else {
            const bindingSuperClass = binding.path.get("superClass");
            if (isPathReactClass(bindingSuperClass)) {
                answer = true;
            }
        }
    }
    return answer;
}
exports.isReactClass = isReactClass;
//# sourceMappingURL=util.js.map